#define STRICT
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "TransXFunction.h"
#include "transx.h"

TransXFunction::TransXFunction(class transxstate *tstate, OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase)
{	
	state=tstate;
	sethandles(thmajor,thminor,thtarget,thcraft,thbase);
	simstartMJD=oapiTime2MJD(0);
}

TransXFunction::~TransXFunction()
{}

bool initialisevars() //Virtual function overloaded by other functions
{
	return true;
}  

void TransXFunction::saveself(FILEHANDLE scn)
{
	//Write terminator
	oapiWriteScenario_string(scn,"Finish","TransXFunction");
} //Should never be used - overloaded by further functions

void TransXFunction::restoreself(FILEHANDLE scn)
{
	//Search for the terminator, then return
	char *bufferpointer;
	char tempbuffer[18], finalbuffer[18];
	bool ok;
	do {
		ok=oapiReadScenario_nextline(scn,bufferpointer);
		strncpy(tempbuffer,bufferpointer,16);
		sscanf(tempbuffer,"%s",finalbuffer);
	}
	while (strcmp(finalbuffer,"Finish")!=0 && ok==true);
}

bool TransXFunction::loadhandle(FILEHANDLE scn,OBJHANDLE *handle)
{
	char *member,*bufferpointer;
	int length;
	bool ok=oapiReadScenario_nextline(scn,bufferpointer);
	if (!ok) return false;//Unexpected end of file found
	parser.parseline(bufferpointer);
	if (!parser.getlineelement(0,&member,&length)) return false;
	if (strcmp(member,"Finish")==0) return false;
	if (strcmp(member,"Handle")!=0) return true;
	if (!parser.getlineelement(1,&member,&length)) return false;//Field missing
	if (strcmp(member,"NULL")==0)
	{
		*handle=NULL;
		return true;//Handle set null
	}
	OBJHANDLE thandle=oapiGetObjectByName(member);
	if (thandle!=NULL) *handle=thandle;
	return true;
}


void TransXFunction::findfinish(FILEHANDLE scn)
{
	char *tbuffer,*member;
	int length;
	do
	{
		if (!oapiReadScenario_nextline(scn,tbuffer)) return;
		parser.parseline(tbuffer);
		parser.getlineelement(0,&member, &length);
	}
	while (strcmp(member,"Finish"));
}


void TransXFunction::savedouble(FILEHANDLE scn, double savenumber)
{
	char buffer[80];
	sprintf(buffer," %g",savenumber);
	oapiWriteScenario_string(scn,"Double",buffer);
}

void TransXFunction::savevector(FILEHANDLE scn, VECTOR3 &vector)
{
	char buffer[100];
	sprintf(buffer," %g %g %g",vector.x,vector.y,vector.z);
	oapiWriteScenario_string(scn,"Vector",buffer);
}


bool TransXFunction::loaddouble(FILEHANDLE scn, double *loadednumber)
{
	char *member,*bufferpointer;
	int length;
	bool ok=oapiReadScenario_nextline(scn,bufferpointer);
	if (!ok) return false;//Unexpected end of file found
	parser.parseline(bufferpointer);
	if (!parser.getlineelement(0,&member,&length)) return false;
	if (strcmp(member,"Finish")==0) return false; //Unexpected end of function found
	if (strcmp(member,"Double")!=0) return true; //Not a double
	if (!parser.getlineelement(1,&member,&length)) return false;
	*loadednumber=atof(member);
	return true;
}

bool TransXFunction::loadint(FILEHANDLE scn, int *loadedint)
{
	char *member,*bufferpointer;
	int length;
	bool ok=oapiReadScenario_nextline(scn,bufferpointer);
	if (!ok) return false;//Unexpected end of file found
	parser.parseline(bufferpointer);
	if (!parser.getlineelement(0,&member,&length)) return false;
	if (strcmp(member,"Finish")==0) return false; //Unexpected end of function found
	if (strcmp(member,"Int")!=0) return true; //Not an Int
	if (!parser.getlineelement(1,&member,&length)) return false;
	sscanf(member," %i",loadedint);
	return true;
}


bool TransXFunction::loadvector(FILEHANDLE scn, VECTOR3 *loadedvector)
{
	char *bufferpointer,*member;
	int length;
	bool ok=oapiReadScenario_nextline(scn,bufferpointer);
	if (!ok) return false;//Unexpected end of file found
	parser.parseline(bufferpointer);
	if (!parser.getlineelement(0,&member,&length)) return false;
	if (strcmp(member,"Finish")==0) return false; //Unexpected end of function found
	if (strcmp(member,"Vector")!=0) return true; //Not a vector
	if (!parser.getlineelement(1,&member,&length)) return false;
	(*loadedvector).x=atof(member);
	if (!parser.getlineelement(2,&member,&length)) return false;
	(*loadedvector).y=atof(member);
	if (!parser.getlineelement(3,&member,&length)) return false;
	(*loadedvector).z=atof(member);
	return true;
}


bool TransXFunction::loadorbit(FILEHANDLE scn,ORBIT *loadorbit)
{
	char *bufferpointer, *member;
	bool ok=oapiReadScenario_nextline(scn,bufferpointer);
	int length;
	if (!ok) return false;
	parser.parseline(bufferpointer);
	if (!parser.getlineelement(0,&member,&length)) return false;//No fields
	if (strcmp(member,"Orbit")!=0) return false; //Not an orbit
	if (!parser.getlineelement(1,&member,&length)) return false;//No second field
	if (strcmp(member,"False")==0)
	{
		//Uninitialised orbit structure
		loadorbit->setinvalid();
		return true;
	}
	VECTOR3 tpos,tvel;
	double gmplanet, timestamp;
	if (!loadvector(scn,&tpos)) return false;
	if (!loadvector(scn,&tvel)) return false;
	if (!loaddouble(scn,&gmplanet)) return false;
	if (!loaddouble(scn,&timestamp)) return false;
	timestamp=(timestamp-simstartMJD)*86400;
	loadorbit->init(tpos,tvel,timestamp,gmplanet);
	return true;
}

void TransXFunction::savehandle(FILEHANDLE scn, OBJHANDLE handle)
{
	char namebuffer[30];
	if (handle!=NULL)
	{
		oapiGetObjectName(handle,namebuffer,30);
	}
	else
	{
		strcpy(namebuffer,"NULL");
	}
	oapiWriteScenario_string(scn,"Handle",namebuffer);
	return;
}


void TransXFunction::saveorbit(FILEHANDLE scn, const ORBIT &saveorbit)
{
	char validvalue[6];
	if (saveorbit.isvalid())
	{
		strcpy(validvalue,"True");
	}
	else
	{
		strcpy(validvalue,"False");
	}
	oapiWriteScenario_string(scn,"Orbit",validvalue);
	if (!saveorbit.isvalid()) return;
	VECTOR3 tpos,tvel;
	saveorbit.getcurrentvectors(&tpos,&tvel);
	savevector(scn,tpos);
	savevector(scn,tvel);
	double planet=saveorbit.getgmplanet();
	savedouble(scn,planet);
	double time=saveorbit.gettimestamp()/86400+simstartMJD;
	savedouble(scn,time);
}










int TransXFunction::getviewmode()
{
	return viewmode;
}

void TransXFunction::setviewmode(int tviewmode)
{
	viewmode=tviewmode;
}

void TransXFunction::setnumberviews(int tnumberviews)
{
	numberviews=tnumberviews;
}

MFDvarhandler *TransXFunction::getvariablehandler()
{
	return &vars;
}

MFDvariable *TransXFunction::getcurrentvariable()
{
	return vars.getcurrent(viewmode);
}



void TransXFunction::inc_viewmode()
{
	if (++viewmode>numberviews) viewmode=1;
}

void TransXFunction::sethmajor(OBJHANDLE handle)
{
	hmajor=handle;
	hminor=hmajtarget=NULL;
}

bool TransXFunction::sethminor(OBJHANDLE handle)
{
	if (hmajor==handle) return false;
	if (handle==hmajtarget) hmajtarget=NULL;
	hminor=handle;
	if (hminor==NULL) return false;
	gravbodyratio=pow(oapiGetMass(hminor)/oapiGetMass(hmajor), double (0.8));
	return true;
}

bool TransXFunction::sethmajtarget(OBJHANDLE handle)
{
	if (hmajor==handle) return false;
	if (hminor==handle) hminor=NULL;
	hmajtarget=handle;
	return true;
}

void TransXFunction::sethcraft(OBJHANDLE handle)
{
	hcraft=handle;
}

void TransXFunction::sethbase(OBJHANDLE handle)
{
	hbase=handle;
}

void TransXFunction::sethandles(OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase)
{
	hmajor=thmajor;
	hminor=thminor;
	gravbodyratio=0;
	if (hminor!=NULL && hmajor!=NULL)
	{
		gravbodyratio=pow(oapiGetMass(hminor)/oapiGetMass(hmajor), double (0.8));
	}
	hmajtarget=thtarget;
	hcraft=thcraft;
	hbase=thbase;
}


void TransXFunction::gethandles(OBJHANDLE *thmajor, OBJHANDLE *thminor, OBJHANDLE *thtarget, OBJHANDLE *thcraft, OBJHANDLE *thbase)
{
	*thmajor=hmajor;
	*thminor=hminor;
	*thtarget=hmajtarget;
	*thcraft=hcraft;
	*thbase=hbase;
}

HPEN TransXFunction::SelectDefaultPen(HDC hDC, int value)
{
	return state->GetMFDpointer()->SelectDefaultPen(hDC,value);
}

bool TransXFunction::encounter (HDC hDC, GRAPH &locgraph, OBJHANDLE hobject, OBJHANDLE satellitebodyhandle, INTERCEPT *intercept, DWORD pencolour)
//Doesn't yet do returnable time accounting
{
	VESSELSTATUS vstatus;
	VECTOR3 irelpos,irelvel;
	intercept->getrelpos(&irelpos);
	intercept->getrelvel(&irelvel);
	VECTOR3 ecliptic={0,-1,0};
	if (hmajtarget==NULL) return false;
	VECTOR3 targetposition, //Current position of target
		relpos,relvel; //current relative position and velocity!
	double targetmass,majorbodymass,ped, timeoffset;
	HPEN pen;
	oapiGetRelativePos(hmajtarget,hmajor,&targetposition);
	oapiGetRelativePos(hobject, hmajtarget,&relpos);
	oapiGetRelativeVel(hobject,hmajtarget,&relvel);//Gets present position and velocity relative to target
	targetmass=oapiGetMass(hmajtarget);
	majorbodymass=oapiGetMass(hmajor);
	double soisize2=pow(targetmass/majorbodymass,0.8)*dotproduct(targetposition,targetposition);
	if (dotproduct(irelpos,irelpos)>soisize2 && dotproduct(relpos,relpos)>soisize2) return false;
	//Inside the soi - time to calculate the orbit inside
	
	//Find out if this is a bound orbit or not
	ORBIT insoi;
	if (GRAVITY*targetmass/vectorsize(relpos)>dotproduct(relvel,relvel)/2)
	{
		//This is a bound orbit
		insoi.init(hmajtarget,hobject);
		locgraph.setviewscale(0);
		locgraph.setviewscale(insoi);
		locgraph.setprojection(insoi);
		pen=SelectDefaultPen(hDC,pencolour);
		insoi.draworbit(hDC,&locgraph,true);//Draw the orbit
		pen=SelectDefaultPen(hDC,5);
		locgraph.drawplanet(hDC,hmajtarget);
		timeoffset=insoi.getpedeltatime();
	}
	else
	{
		//This is an unbound orbit
		//Find out if periapsis is in front of us, or behind
		if (dotproduct(relpos,relvel)<0)
		{
			//We are travelling inward - compensation estimate required!
			//We need to do a straight line extrapolation back to either the current
			// radius, or the SOI, whichever is closer
			double backradius2,temp;
			temp=dotproduct(relpos,relpos);
			if (temp>soisize2*10)
			{
				backradius2=soisize2*10;
				timeoffset=2;
			}
			else
			{
				backradius2=temp;
				timeoffset=0;
			}
			backradius2-=dotproduct(irelpos,irelpos);//Pythagoras - calculates length of backtrack vector
			if (backradius2<0) backradius2=0; //Guard against numerical error
			VECTOR3 backtrack=unitise(irelvel)*sqrt(backradius2);//Calculate backtrack
			VECTOR3 comppos=irelpos-backtrack;
			insoi.init(comppos,irelvel,GRAVITY*targetmass);//Compensated orbit
			locgraph.setviewscale(0);
			locgraph.setviewscalesize(insoi.getpedistance()*2);
			locgraph.setviewscalesize(oapiGetSize(hmajtarget));
			locgraph.setprojection(insoi);
			pen=SelectDefaultPen(hDC,pencolour);
			insoi.draworbit(hDC,&locgraph,false);//Draw the orbit
			pen=SelectDefaultPen(hDC,5);
			locgraph.drawplanet(hDC,hmajtarget);
			if (timeoffset<1)
			{
				timeoffset=insoi.getpedeltatime();
			}
			else
			{
				double timeintercept=intercept->gettimeintercept();
				timeoffset=timeintercept-oapiGetSimTime();
			}
		}
		else
		{
			//We are travelling outward - just give the orbit
			insoi.init(hmajtarget,hobject);
			locgraph.setviewscale(0);
			locgraph.setviewscale(insoi);
			locgraph.setprojection(insoi);
			pen=SelectDefaultPen(hDC,pencolour);
			insoi.draworbit(hDC,&locgraph,true);//Draw the orbit
			pen=SelectDefaultPen(hDC,5);
			locgraph.drawplanet(hDC,hmajtarget);
			timeoffset=insoi.getpedeltatime();
		}
	}
	//Draw the intersection of the satellite plane with the craft plane
	VECTOR3 satpos,satvel,satplane,satintersect;
	//Get the satellite body handle
	if (satellitebodyhandle!=NULL)
	{
		oapiGetRelativePos(hmajtarget,satellitebodyhandle,&satpos);
		oapiGetRelativeVel(hmajtarget,satellitebodyhandle,&satvel);
		satplane=crossproduct(satpos,satvel);
		satintersect=crossproduct(satplane,insoi.getplanevector());
		locgraph.drawvector(hDC,satintersect);
	}


	TextShow(hDC,"Arr.T:",0,3*linespacing,timeoffset);
	//We now have an orbit insoi, which may (or may not) be compensated, but represents best
	//case approach estimate
	ped=insoi.getpedistance();
	DWORD hpos=4*linespacing;
	TextShow(hDC,"Pe D:",0,hpos,ped);
	hpos+=linespacing;
	double planetradius=oapiGetSize(hmajtarget);
	TextShow(hDC,"Maj. Rad:",0,hpos,planetradius);
	hpos+=linespacing;
	if (planetradius<ped)
	{
		TextShow(hDC,"Min Alt:",0,hpos,ped-planetradius);
		hpos+=linespacing;
		double planetenergy=GRAVITY*oapiGetMass(hmajtarget)/ped;
		double escvelocity=sqrt(2*planetenergy);
		double actualpevelocity=sqrt(insoi.getangmomentum2())/ped;
		TextShow(hDC,"Pe Vel:",0,hpos,actualpevelocity);
		hpos+=linespacing;
		TextShow(hDC,"Capture Dv:",0,hpos,actualpevelocity-escvelocity);
		hpos+=linespacing;
	}
	double inclination=acos(dotproduct(unitise(insoi.getplanevector()),ecliptic));
	inclination=inclination/PI*180;
	TextShow(hDC,"Inc:",0,hpos,inclination);
	hpos+=linespacing;
	VECTOR3 lanvector=unitise(crossproduct(insoi.getplanevector(),ecliptic));
	double lan=acos(lanvector.x)/PI*180;
	if (lanvector.z<0) lan=-lan;
	TextShow(hDC,"LAN ",0,hpos,lan);
	hpos+=linespacing+linespacing;
	//Attempt to get a surface base reference
	VESSEL *pV=oapiGetFocusInterface();
	pV->GetStatus(vstatus);
	OBJHANDLE tbasehandle=vstatus.base;
	if (tbasehandle!=NULL && hbase!=tbasehandle)
	{
		VECTOR3 baseoffset=targetposition;
		oapiGetRelativePos(tbasehandle,hmajtarget,&baseoffset);
		if (vectorsize(baseoffset)<planetradius*1.01)//Only use basehandle if actually a surface base
		{
			hbase=tbasehandle;
		}
	}
	if (hbase!=NULL)
	{
		VECTOR3 baseoffset;
		oapiGetRelativePos(hbase,hmajtarget,&baseoffset);
		if (vectorsize(baseoffset)>planetradius*1.01) return true;
		pen=SelectDefaultPen(hDC,3);
		locgraph.drawtwovector(hDC,baseoffset,baseoffset);
		double offplane=-dotproduct(baseoffset,unitise(insoi.getplanevector()));
		TextShow(hDC, "Orbit pl. to base:",0,hpos,offplane);
	}
	return true;
}


void TransXFunction::sethelp(char *help1,char *help2,char *help3,char *help4,char *help5)
{
	strcpy(helpstring1,help1);
	strcpy(helpstring2,help2);
	strcpy(helpstring3,help3);
	strcpy(helpstring4,help4);
	strcpy(helpstring5,help5);
}

void TransXFunction::gethelp(char *help1,char *help2,char *help3,char *help4,char *help5) const
{
	strcpy(help1,helpstring1);
	strcpy(help2,helpstring2);
	strcpy(help3,helpstring3);
	strcpy(help4,helpstring4);
	strcpy(help5,helpstring5);
}

