#define STRICT
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "TransXFunction.h"
#include "mfd.h"
#include "cruisefunction.h"
#include "transx.h"


CruiseFunction::CruiseFunction(class transxstate *tstate, OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase)
: TransXFunction(tstate, thmajor, thminor, thtarget, thcraft, thbase)
{ 
	if (!(valid=initialisevars()))
	{
		state->setmfdstate(0);
	}
}

CruiseFunction::~CruiseFunction()
{
	vars.deleteall();
}


void CruiseFunction::saveself(FILEHANDLE scn)
{
	vars.saveallvariables(scn);
	saveorbit(scn,basisorbit);
	savehandle(scn,hmajor);
	savehandle(scn,hminor);
	savehandle(scn,hmajtarget);
	oapiWriteScenario_int(scn,"Int",getviewmode());
	oapiWriteScenario_string(scn,"Finish","CruiseFunction");
}

void CruiseFunction::restoreself(FILEHANDLE scn)
{
	OBJHANDLE temp;
	if (!vars.loadallvariables(scn)) return;
	if (!loadorbit(scn,&basisorbit)) return;
	if (!loadhandle(scn,&temp)) return;
	if (temp!=NULL) sethmajor(temp);//hmajor must never be set null
	if (!loadhandle(scn,&temp)) return;
	sethminor(temp);
	if (!loadhandle(scn,&hmajtarget)) return;
	int temp2;
	if (!loadint(scn, &temp2)) return;
	setviewmode(temp2);
	findfinish(scn);
}




bool CruiseFunction::initialisevars()
{
	enum mode2variables {orbitviews,base_orbit,progradevel,ejdate,outvel,chplvel,projection,intercept,sathandle};
	vars.deleteall();
	bool test=true;
	test&=vars.newvardiscrete("View Orbits",0,1,"All","Target","","","");
	test&=vars.newvardiscrete("Base Orbit",1,1,"++ Updates","Updating","","","");
	test&=vars.newvarfloat("Prograde vel.", 1, 0, -1e8, 1e8, 0.05, 1000);
	test&=vars.newvarMJD("Eject date", 1, 0, 0, 1e20, 0.000005, 1000000);
	test&=vars.newvarfloat("Outward vel.", 1, 0,-1e8,1e8,0.05,1000);
	test&=vars.newvarfloat("Ch. plane vel.", 1, 0, -1e8, 1e8, 0.05,1000);
	test&=vars.newvardiscrete("Projection",0,1, "Ecliptic","Craft","","","");
	test&=vars.newvardiscrete("Intercept with",0,1,"Craft","Hypothetical","","","");
	test&=vars.newvarbody("Satellite","Select Satellite of Target");
	if (test==false)
	{
		vars.deleteall();
		valid=false;
		return false;//Occurs when for some reason mfdvariables are NOT created
	}
	vars.setviewrange(1,0,0);
	vars.setviewrange(2,0,8);
	vars.setviewrange(3,8,9);

	setnumberviews(3);
	setviewmode(2);
	valid=true;
	//Set help system up


	vars.sethelpstrings(orbitviews,
		"Set to 'Target' to obtain better",
		"view when target is small");
	vars.sethelpstrings(base_orbit,
		"Orbit on which manoevres",
		"are based. ++ sets to craft orbit");
	vars.sethelpstrings(progradevel,
		"Positive numbers to outer planets.",
		"Negative numbers to inner planets.");
	vars.sethelpstrings(ejdate,
		"Your planned launch date/time",
		"Alter to seek good launch windows");
	vars.sethelpstrings(outvel,
		"Add velocity outwards/inwards.",
		"Allows launch date flexibility.");
	vars.sethelpstrings(chplvel,
		"Use to swing grey plane intersection",
		"line towards intercept point.");
	vars.sethelpstrings(projection,
		"Adjusts the way MFD displays orbit",
		"Adjust to taste");
	vars.sethelpstrings(intercept,
		"Intercept info shown for whatever",
		"you select here.");
	vars.sethelpstrings(sathandle,
		"Select Satellite of the Target.Shows",
		"where orbital plane intersects yours");
	sethelp(
		"Used when in the sphere of influence",
		"of major body only, en route to a",
		"target. Contains course correction",
		"tools and close view of encounter",
		"with target.");
	return true;
}


bool CruiseFunction::doupdate(HDC hDC, int tw, int th)
// This is the update mode for when craft is outside RMin SOI
{
	if (!valid) return true;
	//Get current graph parameters from current MFD
	graph=state->GetMFDpointer()->graph;
	W=tw;
	H=th;
	linespacing=H/20;
	hypormaj.setinvalid();
	VECTOR3 gmajorpos, gminorpos, gcraftpos;
	VECTOR3 temp={0,0,0};
	double soisize2;
	enum mode2variables {orbitviews,base_orbit,progradevel,ejdate,outvel,chplvel,projection,intercept,sathandle};
	// Get positions of major, minor bodies and spacecraft
	oapiGetGlobalPos(hmajor, &gmajorpos);
	oapiGetGlobalPos(hcraft, &gcraftpos);
	if (hminor!=NULL) 
	{
		oapiGetGlobalPos(hminor, &gminorpos);
		temp=gmajorpos-gminorpos;
		// Set the size of the sphere of influence
		soisize2=dotproduct(temp,temp)*gravbodyratio;
		temp=gcraftpos-gminorpos;
		if (dotproduct(temp,temp)<soisize2) // Craft is inside SOI
		{
			if (state->mfdstatevalidated) //It's been OK - autoswitch
			{
				state->setmfdstate(1);
				return false;
			}
			else
			{
				int wpos=0;
				int hpos=4*linespacing;
				TextOut(hDC,wpos, hpos, "Function not valid",18);
				hpos+=linespacing+linespacing;
				TextOut(hDC,wpos,hpos, "Inside Minor SOI",16);
				// Label the mode
				wpos=W/2;
				hpos=0;
				TextOut(hDC,wpos, hpos, "Fnctn:Cruise",12);
				return true;
			}
		}
	}

	state->mfdstatevalidated=true;//Deprecated

	// Check whether craft is in the right mode, and switch if it is not


	// Set the projection to ecliptic
	VECTOR3 ntemp={0,-1,0};
	graph.setprojection(ntemp);

	// Initialise orbit info for craft and minor
	craft.init(hmajor, hcraft);
	rmin.init(hmajor, hminor);
	//Initialise orbit info for target 
	target.init(hmajor, hmajtarget);

	if (!basisorbit.isvalid() || vars.getivalue(base_orbit)==1)
	{
		basisorbit=craft;
		vars.setivalue(base_orbit,0);
	}
	VECTOR3 targetvel,craftpos,craftvel;
	Getmode2hypo(&targetvel);
	if (getviewmode()==1 && !hypormaj.isvalid()) setviewmode(2);
	if (getviewmode()==1)
	{
		double timeoffset=(vars.getvalue(ejdate)-simstartMJD)*86400-craft.gettimestamp();
		craft.timetovectors(timeoffset,&craftpos,&craftvel);
		VESSEL *pV=oapiGetFocusInterface();
		double rvel=graph.vectorpointdisplay(hDC, targetvel-craftvel, state->GetMFDpointer(), pV, false);
		int wpos=W/2;
		int hpos=0;
		TextOut(hDC,wpos,hpos, "Fnctn:Cruise",12);
		hpos=linespacing;
		TextOut(hDC,wpos,hpos,"View:Vector",11);
		TextShow(hDC,"Rel V: ",0,18*linespacing,rvel);
		TextShow(hDC,"T to Mnvre ",0,19*linespacing,timeoffset);
	}
	else
	{
		double scale=0;	
		// Set the viewscale for the graph
		graph.setviewscale(scale);
		if (vars.getivalue(orbitviews)==0 || !target.isvalid()) //Only set viewscale to view these if user desires it
		{
			graph.setviewscale(craft);
			graph.setviewscale(rmin);
		}
		bool interceptflag=false;
		bool shownencounterview=false;
		OBJHANDLE satbodyhandle=vars.gethandle(sathandle);
		// If appropriate, calculate the intersection elements for the target
		if (target.isvalid())
		{
			graph.setviewscale(target);
			if (vars.getivalue(intercept)==0)
			{
				primary.updateintercept(craft, target);
				interceptflag=true;
				if (getviewmode()==3)
				{
					shownencounterview=encounter(hDC,graph, hcraft,satbodyhandle,&primary,0);
					if (!shownencounterview) setviewmode(1);
				}
			}
		}
		if (hypormaj.isvalid())
		{
			if (vars.getivalue(orbitviews)==0) graph.setviewscale(hypormaj);
			if (vars.getivalue(intercept)==1 && target.isvalid())
			{
				primary.updateintercept(hypormaj, target);
				interceptflag=true;
				if (getviewmode()==3)
				{
					shownencounterview=encounter(hDC,graph, hcraft,satbodyhandle,&primary,3);
					if (!shownencounterview) setviewmode(1);
				}
			}
		}

		// Label the mode
		int wpos=W/2;
		int hpos=0;
		TextOut(hDC,wpos,hpos, "Fnctn:Cruise",12);
		hpos=linespacing;
		if (shownencounterview)
		{
			TextOut(hDC,wpos,hpos, "View:Encounter",14);
			return true;
		}
		else
		{
			TextOut(hDC,wpos, hpos, "View:Major",10);
		}

		if (shownencounterview) return true;
		// Draw the craft orbit
		HPEN pen;
		DWORD i=1;
		pen=SelectDefaultPen(hDC,i);
		graph.draworbit(craft, hDC, true);

		//Draw the minor object orbit
		i=2;
		pen=SelectDefaultPen(hDC,i);
		graph.draworbit(rmin, hDC, true);
		if (hypormaj.isvalid())
		{
			i=3;
			pen=SelectDefaultPen(hDC,i);
			graph.draworbit(hypormaj,hDC,true);
		}
		i=5;
		pen=SelectDefaultPen(hDC,i);
		graph.drawplanet(hDC,hmajor); //Draw the central planet(star)
		// If there is a target, draw it, and the targeting lines
		if (target.isvalid())
		{
			i=2;
			pen=SelectDefaultPen(hDC,i);
			graph.draworbit(target, hDC, true);
			if (interceptflag)
			{
				i=3;
				pen=SelectDefaultPen(hDC,i);
				VECTOR3 craftpos, targetpos, intersect;
				primary.getpositions(&craftpos,&targetpos);
				graph.drawtwovector(hDC, craftpos,targetpos);
				primary.getplanecept(&intersect);
				i=5;
				pen=SelectDefaultPen(hDC,i);
				graph.drawvector(hDC,intersect);
				char buffer[20];
				int hpos=3*linespacing;
				int wpos=0;
				int length=sprintf(buffer, "Cl. App.");
				TextOut(hDC, wpos, hpos, buffer, length);
				hpos+=linespacing;
				TextShow(hDC, " ", wpos, hpos, vectorsize(craftpos-targetpos));
				hpos+=linespacing;
				double size=oapiGetSize(hmajtarget);
				TextShow(hDC,"Tgt R ",wpos,hpos,size);
				hpos+=linespacing;
				VECTOR3 relvel;
				primary.getrelvel(&relvel);
				TextShow(hDC,"Enc. V:", wpos, hpos, vectorsize(relvel));
				hpos+=linespacing;
				double intercepttime=primary.gettimeintercept();
				double arrmjd=intercepttime/86400+simstartMJD;
				length=sprintf(buffer,"Enc. MJD %.2f", arrmjd);
				TextOut(hDC, wpos, hpos, buffer, length);
			}
		}
		else
		{
			int hpos=6*linespacing;
			int wpos=0;
			TextShow(hDC,"Maj. Rad:",wpos,hpos,oapiGetSize(hmajor));
			hpos+=linespacing;
			TextShow(hDC,"Craft PeD:", wpos, hpos, craft.getpedistance());
			hpos+=linespacing;
			char buffer[20];
			int length=sprintf(buffer,"Pe MJD %.2f",(craft.getpedeltatime()+craft.gettimestamp())/86400+simstartMJD);
			TextOut(hDC,wpos,hpos,buffer, length);
			hpos+=linespacing;
			if (hypormaj.isvalid())
			{
				TextShow(hDC,"Hyp Ped ",wpos,hpos,hypormaj.getpedistance());
				hpos+=linespacing;
				double timeatped=(hypormaj.gettimestamp()+hypormaj.getpedeltatime())/86400+simstartMJD;
				length=sprintf(buffer,"H. Pe MJD %.2f",timeatped);
				TextOut(hDC,wpos,hpos,buffer,length);
			}
		}
	}
	return true;
}


void CruiseFunction::Getmode2hypo(VECTOR3 *targetvel)

// This obtains the current hypothetical orbit in rmin influence

{
	enum mode2variables {orbitviews,base_orbit,progradevel,ejdate,outvel,chplvel,projection,intercept,sathandle};

	// New section
	double mjd=vars.getvalue(ejdate);//Gets the mjd for eject
	double difftime=(mjd-simstartMJD)*86400-basisorbit.gettimestamp();//Converts to seconds from present time
	VECTOR3 ejradius, ejvel;
	basisorbit.timetovectors(difftime, &ejradius, &ejvel);
	VECTOR3 forward=unitise(ejvel)*vars.getvalue(progradevel);
	VECTOR3 outward=unitise(crossproduct(ejvel, basisorbit.getplanevector()))*vars.getvalue(outvel);
	VECTOR3 sideward=unitise(basisorbit.getplanevector())*vars.getvalue(chplvel);
	VECTOR3 ejectvector=forward+outward+sideward; //=Eject vector in basisorbit frame
	double ejvelocity=vectorsize(ejectvector);
	if (ejvelocity<0.01) return;

	VECTOR3 hypopos, hypovel;
	// Add to basisorbit vectors at eject time to give ejection vector in rmaj
	hypopos=ejradius; 
	hypovel=ejectvector+ejvel; 
	*targetvel=hypovel;

	//Create hypothetical orbit in rmaj
	hypormaj.init(hypopos, hypovel, (mjd-simstartMJD)*86400, basisorbit.getgmplanet());
}


