#define STRICT
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "TransXFunction.h"
#include "mfd.h"
#include "ejectfunction.h"
#include "transx.h"


EjectFunction::EjectFunction(class transxstate *tstate, OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase)
: TransXFunction(tstate, thmajor, thminor, thtarget, thcraft, thbase)
{
	if (!(valid=initialisevars()))
	{
		state->setmfdstate(0);
	}
}

EjectFunction::~EjectFunction()
{
	vars.deleteall();

}


void EjectFunction::saveself(FILEHANDLE scn)
{
	vars.saveallvariables(scn);
	savehandle(scn,hmajor);
	savehandle(scn,hminor);
	savehandle(scn,hmajtarget);
	oapiWriteScenario_int(scn,"Int",getviewmode());
	oapiWriteScenario_string(scn,"Finish","EjectFunction");
}


void EjectFunction::restoreself(FILEHANDLE scn)
{
	OBJHANDLE temp;
	if (!vars.loadallvariables(scn)) return;
	if (!loadhandle(scn,&temp)) return;
	if (temp!=NULL) sethmajor(temp);//hmajor must never be set null
	if (!loadhandle(scn,&temp)) return;
	sethminor(temp);
	if (!loadhandle(scn,&hmajtarget)) return;
	int temp2;
	if (!loadint(scn,&temp2)) return;
	setviewmode(temp2);
	findfinish(scn);
}




bool EjectFunction::doupdate(HDC hDC, int tw, int th)
// This is the eject mode. It has two views, major and minor centred
// It covers the case that the craft is inside the RMIN sphere of influence
{
// Get the positions of the major relevant bodies in the global frame
	if (!valid) return true;
	graph=state->GetMFDpointer()->graph;
	W=tw;//Set width and height parameters of MFD for this update
	H=th;
	linespacing=H/20;
	enum mode1variables {
		periapsis, ejorient, minprojection, minvieworb, ejprovel, ej_mjd, ejoutvel, ejincvel, majprojection, hypoorcraft, majvieworb};
	VECTOR3 gcraftpos, gmajorpos,temp;
	VECTOR3 gminorpos={1,1,1};//Just a value to initialise it to something

	// Get positions of bodies

	oapiGetGlobalPos(hmajor, &gmajorpos);
	if (hminor!=NULL)
	{
		oapiGetGlobalPos(hminor, &gminorpos);
		gmminor=oapiGetMass(hminor)*GRAVITY;
	}
	oapiGetGlobalPos(hcraft, &gcraftpos);
	temp=gmajorpos-gminorpos;
	soisize2=dotproduct(temp,temp)*gravbodyratio;
	bool invalidated;
	bool rettype=minormodevalidate(hDC,gminorpos,gcraftpos, "Fnctn:Eject",2,&invalidated);
	if (invalidated) return rettype;
	state->mfdstatevalidated=true;//All criteria needed to display this mode fulfilled

	//Craft now known to be inside SOI

	//Check if variables are initialised - if not, initialise them

	//Initialise orbit of minor around major
	rmin.init(hmajor,hminor);
	craftinrmin.init(hminor,hcraft);
	hypormin.setinvalid();
	hypormaj.setinvalid();
	craftinrmaj.setinvalid();


	//This section calculates the hypothetical ejection path.
	Getmode1hypo();

	if (getviewmode()==1)
	{
		// This is the RMin centred view
		// Compute an equatorial projection
		if (vars.getivalue(minprojection)==3)
		{
			double longitude, latitude, tnumber;
			BOOL noerrflg; //Need this type of bool to match SDK
			VECTOR3 tvelocity, tvector, t2vector, tradius;
			noerrflg=oapiGetFocusShipAirspeedVector(&tvector);//Vector in ship frame - not what I want
			VESSEL *pV=oapiGetFocusInterface();
			pV->GlobalRot(tvector,t2vector);// t2vector now in global frame
			if (noerrflg)//Now works with moving ships too
			{
				oapiGetFocusRelativeVel(hminor, &tvelocity); //This is the velocity under the ship
				tvelocity=tvelocity-t2vector; //This is the velocity of the surface
				tvelocity=unitise(tvelocity); //Unitise to give vector parallel to rotation
				craftinrmin.getcurrentvectors(&tradius,&tvector);
				tradius=unitise(tradius);
				tvector=crossproduct(tradius, tvelocity);
				oapiGetFocusEquPos(&longitude, &latitude, &tnumber);//Only works if current planet's surface dominates
				double cosl=cos(-latitude);
				double sinl=sin(-latitude);
				VECTOR3 tzaxis=unitise(tradius)*cosl-unitise(tvector)*sinl;
				VECTOR3 tyaxis=unitise(tradius)*sinl+unitise(tvector)*cosl;
				graph.setprojection(tvelocity, tyaxis,tzaxis);
			}
			else
			{
				graph.setprojection(temp);//If all else fails, use ecliptic
			}
		}
		showminorsoiview1(hDC, &graph, craftinrmin, hypormin, minprojection, minvieworb, true);
	}
	else
	{
	// Calculate and show RMaj centred view

		if (craftinrmin.geteccentricity()>1) // There's a craft orbit in RMaj
		{
			// This gives us the craft's orbit in rmaj
			craftinrmin.minortomajor(rmin, &craftinrmaj, sqrt(soisize2));
		}

		// Get the orbit of the target, if target set
		majtarget.init(hmajor, hmajtarget);
		

		// Get the correct intercept 
		bool showintercept;
		bool h_or_c=false;
		if (vars.getivalue(hypoorcraft)==0) h_or_c=true;

		showintercept=updateinterception(hypormaj,craftinrmaj,majtarget,h_or_c);

		//Show the information gathered
		showminorsoiview2(hDC, &graph, h_or_c, showintercept, majprojection, majvieworb, false);

	}
	// Label the mode
	int wpos=W/2;
	int hpos=0;
	TextOut(hDC,wpos, hpos, "Fnctn:Eject",11);
	return true;
}


bool EjectFunction::initialisevars()

// Initialise MFDvariable objects used by mode 1
{
	enum mode1variables {
	periapsis, ejorient, minprojection, minvieworb, ejprovel, ej_mjd, ejoutvel, ejincvel, majprojection, hypoorcraft, majvieworb};

	vars.deleteall();
	bool test=true;
	double radius;
	if (hminor!=NULL)
	{
		radius=oapiGetSize(hminor);
	}
	else
	{
		radius=1e6;
	}
	test&=vars.newvarfloat("Pe Distance",1, radius*1.2, 0, radius*1000, 0.05, 1000);
	test&=vars.newvarangle("Ej Orientation", 1, 0, 1, 0, -PI, PI, PI/90, true);
	test&=vars.newvardiscrete("Projection",0,3, "Ecliptic", "Craft", "Target","Equatorial","");
	test&=vars.newvardiscrete("View Orbits",0,1,"All","Hypothetical","","","");
	test&=vars.newvarfloat("Prograde vel.", 1, 0, -1e8, 1e8, 0.05, 1000);
	test&=vars.newvarMJD("Eject date", 1, 0, 0, 1e20, 0.000005, 1000000);
	test&=vars.newvarfloat("Outward vel.", 1, 0,-1e8,1e8,0.05,1000);
	test&=vars.newvarfloat("Ch. plane vel.", 1, 0, -1e8, 1e8, 0.05,1000);
	test&=vars.newvardiscrete("Projection",0,1, "Ecliptic","Craft","","","");
	test&=vars.newvardiscrete("Intercept with",0,1,"Hypothetical","Craft","","","");
	test&=vars.newvardiscrete("View Orbits",0,2,"All","Target","Craft","","");
	if (test==false)
	{
		vars.deleteall();
		valid=false;
		return false;
	}
	vars.setviewrange(1,0,4);
	vars.setviewrange(2,4,11);
	setnumberviews(2);
	setviewmode(2);
	valid=true;

	vars.sethelpstrings(periapsis,
		"Periapsis of planned orbit",
		"(Distance from planet centre)");
	vars.sethelpstrings(ejorient,
		"Use to rotate planned orbit around",
		"axis of departure direction.");
	vars.sethelpstrings(minprojection,
		"Adjust your viewpoint for viewing",
		"current MFD graph.");
	vars.sethelpstrings(minvieworb,
		"Sometimes helps you get closer",
		"view of hypothetical orbit");
	vars.sethelpstrings(ejprovel,
		"Positive to move outward from MAJ",
		"Negative to move inward toward MAJ");
	vars.sethelpstrings(ej_mjd,
		"Your planned launch date. Adjust to",
		"find efficient launch window.");
	vars.sethelpstrings(ejoutvel,
		"Adjust to give you launch date",
		"flexibility or fast flight.");
	vars.sethelpstrings(ejincvel,
		"Adjust grey target plane intersect",
		"line to coincide with intercept.");
	vars.sethelpstrings(majprojection,
		"Adjust for viewpoint for viewing",
		"current MFD graph.");
	vars.sethelpstrings(hypoorcraft,
		"Intercept info displayed for",
		"selected item.");
	vars.sethelpstrings(majvieworb,
		"Use to obtain better view of target",
		"if target orbit appears small.");
	sethelp(
		"Used to leave SOI of one planet/moon en",
		"en route for another. eg Earth to",
		"Mars. MAJ is body all orbit. Best",
		"to plan before you take off. Most",
		"efficient way to plan");
	return true;
}



void EjectFunction::Getmode1hypo()

// This obtains the current hypothetical orbit in rmin influence
// rmin guaranteed to be present time

{
	enum mode1variables {
	periapsis, ejorient, minprojection, minvieworb, ejprovel, ej_mjd, ejoutvel, ejincvel, majprojection, hypoorcraft, majvieworb} ;

	// New section
	double mjd=vars.getvalue(ej_mjd);//Gets the mjd for periapsis of eject
	double currmjd=oapiGetSimMJD();// Gets the current mjd
	ejectmjd=mjd;
	double difftime=(mjd-currmjd)*86400;//Converts to seconds from present time
	VECTOR3 ejradius, ejvel;
	rmin.timetovectors(difftime, &ejradius, &ejvel);//Where rmin will be at eject time
	// Get eject vector
	VECTOR3 rminplane=rmin.getplanevector();
	VECTOR3 forward=unitise(ejvel)*vars.getvalue(ejprovel);
	VECTOR3 outward=unitise(crossproduct(ejvel, rminplane))*vars.getvalue(ejoutvel);
	VECTOR3 sideward=unitise(rminplane)*vars.getvalue(ejincvel);
	VECTOR3 ejectvector=forward+outward+sideward; //=Eject vector in RMin frame
	double ejvelocity=vectorsize(ejectvector);
	if (ejvelocity<1) return;

	//Set up temporary coordinate system
	VECTOR3 txaxis=unitise(ejectvector); 
	VECTOR3 tzaxis=unitise(crossproduct(ejectvector, rminplane));
	VECTOR3 tyaxis=unitise(crossproduct(ejectvector, tzaxis));
	double ejorientsinvalue, ejorientcosvalue;
	vars.getsincos(ejorient,&ejorientsinvalue, &ejorientcosvalue);
	VECTOR3 tnaxis=tzaxis*ejorientcosvalue+tyaxis*ejorientsinvalue;

	//Get eccentricity for hypothetical orbit
	double peridistance=vars.getvalue(periapsis);
	if (peridistance*peridistance>soisize2)//Ensures periapsis is not outside sphere of influence.
	{
		peridistance=sqrt(soisize2)*0.9;
		vars.setvalue(periapsis, peridistance);
	}
	double ecc=peridistance*ejvelocity*ejvelocity/gmminor+1; 
	//Get cos and sin thi for infinite radius
	double costhiinfangle=1/ecc;
	double sinthiinfangle=sqrt(1-costhiinfangle*costhiinfangle);

	// Use this, in conjunction with other elements, to obtain 
	//radius and velocity vectors for periapsis of required orbit
	VECTOR3 periposvector=(txaxis*(-costhiinfangle)+tnaxis*sinthiinfangle)*peridistance; //Periapsis position vector
	double tvel=sqrt(2*gmminor/peridistance+ejvelocity*ejvelocity); // Velocity size at periapsis
	VECTOR3 perivelvector=(txaxis*(sinthiinfangle)+tnaxis*costhiinfangle)*tvel; //Velocity vector

	// Create orbit for hypothetical orbit in rmin
	hypormin.init(periposvector, perivelvector, (mjd-simstartMJD)*86400, gmminor);
	// Now need to find the point where this orbit leaves the SOI, and extract vectors
	double exitradius=sqrt(soisize2);
	double timeatsoi=hypormin.GetTimeToRadius(exitradius,true);//outward solution
	VECTOR3 hyporpos, hyporvel, hypopos, hypovel;
	hypormin.radiustovectors( exitradius, true,&hyporpos, &hyporvel);// Gives vectors at exit from SOI
	difftime+=timeatsoi;
	rmin.timetovectors(difftime, &ejradius, &ejvel);//Where rmin will be at exit of soi

	// Add to Min vectors at eject time to give ejection vector in rmaj
	hypopos=hyporpos+ejradius; // Add to Min vector at time of eject to obtain actual ejection vector
	hypovel=ejectvector+ejvel; //Using the original eject vector for velocity - gives velocity at infinity

/* This whole circle of calculations gives a slight refinement of the eject vectors
from those specified - a refinement that takes into account the properties of the chosen ejection orbit */

	//Create hypothetical orbit in rmaj
	hypormaj.init(hypopos, hypovel, difftime, rmin.getgmplanet());
}


bool EjectFunction::minormodevalidate(HDC hDC, const VECTOR3 &gminorpos, const VECTOR3 &gcraftpos, const char *modelabel,int nonvalidmfdstate, bool *invalidated)
{
	*invalidated=true;
	int labellength=strlen(modelabel);
	if (hminor==NULL) //Can't be in hminor SOI if SOI not set.....
	{
		if (state->mfdstatevalidated) //It's been OK - autoswitch
		{
			state->setmfdstate(nonvalidmfdstate);
			return false;
		}
		else
		{
			int wpos=0;
			int hpos=4*linespacing;
			TextOut(hDC,wpos, hpos, "Function not valid",18);
			hpos+=linespacing+linespacing;
			TextOut(hDC,wpos,hpos, "Select a Minor body",19);
			// Label the mode
			wpos=W/2;
			hpos=0;
			TextOut(hDC,wpos, hpos, modelabel,labellength);
			return true;
		}
	}

	VECTOR3 temp=gcraftpos-gminorpos;
	if (dotproduct(temp,temp)>soisize2) // Craft is outside SOI
	{
		if (state->mfdstatevalidated) //It's been OK - autoswitch
		{
			state->setmfdstate(nonvalidmfdstate);
			return false;
		}
		else
		{
			int wpos=0;
			int hpos=4*linespacing;
			TextOut(hDC,wpos, hpos, "Function not valid",18);
			hpos+=linespacing+linespacing;
			TextOut(hDC,wpos, hpos, "Outside Minor SOI",17);
			// Label the mode
			wpos=W/2;
			hpos=0;
			TextOut(hDC,wpos, hpos, modelabel,labellength);
			return true;
		}
	}
	*invalidated=false;//Passed all tests, display remainder.
	return true;
}



void EjectFunction::showminorsoiview1(HDC hDC, GRAPH *locgraph, const ORBIT &craftinrmin, const ORBIT &hypormin, int minprojection, int minvieworb, bool showrinc)
{

	HPEN pen;
	DWORD i;

	// Select projection for this view
	VECTOR3 temp={0,-1,0};
	int projection=vars.getivalue(minprojection);
	switch (projection){
	case 0:
		locgraph->setprojection(temp);
		break;
	case 1:
		locgraph->setprojection(craftinrmin);
		break;
	case 2:
		if (hypormin.isvalid())
		{
			locgraph->setprojection(hypormin);
		}
		else
		{
			locgraph->setprojection(temp);
		}
		break;
	case 3:
		//Use the projection already set!
		break;
	}

	//Set the scaling for the view
	double scale=0;
	locgraph->setviewscale(scale);
	if (vars.getivalue(minvieworb)==0 || !hypormin.isvalid())
	{
		locgraph->setviewscale(craftinrmin);
	}
	// Specific adjustment for this view
	if (craftinrmin.geteccentricity()>1) // Back off view if currently breaking orbit - allows more accurate view of track
	{
		scale=locgraph->getviewscale();
		scale=scale*0.5;
		locgraph->setviewscale(scale);
	}
	if (hypormin.isvalid())
	{
		locgraph->setviewscale(hypormin);

		//Draw line representing intersection between hypothetical orbit and craft orbit
		if (showrinc)
		{
			VECTOR3 intersect=hypormin.getintersectvector(craftinrmin);
			i=5;
			pen=SelectDefaultPen(hDC,i);
			locgraph->drawvector(hDC,intersect);
		}

		// Draw hypothetical orbit
		i=3;
		pen=SelectDefaultPen(hDC,i);
		locgraph->draworbit(hypormin, hDC, true);

		// Print out as text various bits of information
		// Relative inclination
		char buffer[20];
		int length, wpos, hpos;
		wpos=0;
		hpos=2*linespacing;
		if (showrinc)
		{
			VESSELSTATUS status;
			VECTOR3 pos,vel;
			craftinrmin.getcurrentvectors(&pos,&vel);
			double angle=180/PI*acos(dotproduct(unitise(hypormin.getplanevector()),unitise(craftinrmin.getplanevector())));
			VESSEL *pV=oapiGetFocusInterface();
			pV->GetStatus(status);
			if (status.status==1)
			{ //Vessel is landed!
				VECTOR3 tintersectvector=hypormin.getintersectvector(craftinrmin);//Gets intersection vector
				if (dotproduct(tintersectvector,pos)<0)
				{
					angle+=90;
				}
				else
				{
					angle=90-angle;
				}
				if (angle<0) angle+=360;
				length=sprintf(buffer,"Heading:%.4g'", angle);
			}
			else
			{
				length=sprintf(buffer, "R.Inc:%.4g'", angle);
			}
			TextOut(hDC, wpos, hpos, buffer, length);
		}
		// Calculate Delta-v
		VECTOR3 tpos,tvel;
		craftinrmin.getcurrentvectors(&tpos,&tvel);
		double radius=vectorsize(tpos);
		double deltav=sqrt(hypormin.getgmplanet()*(2/radius+1/hypormin.getsemimajor()))-vectorsize(tvel);
		hpos+=linespacing;
		TextShow(hDC,"DeltaV ", wpos, hpos, deltav);

		// Periapsis and Apoapsis distance
		double peri=craftinrmin.getpedistance();
		hpos+=linespacing+linespacing;
		if (craftinrmin.geteccentricity()>1)
		{
			TextShow(hDC,"PeD ", wpos, hpos, peri);
		}
		else
		{
			TextShow(hDC, "PeD ", wpos, hpos, peri);
			hpos+=linespacing;
			double apo=craftinrmin.getapodistance();
			TextShow(hDC, "ApD ", wpos, hpos, apo);
		}

	}
	// Draw the planet
	i=5;
	pen=SelectDefaultPen(hDC,i);
	locgraph->drawplanet(hDC, hminor);
	// Draw craft orbit
	i=1;
	pen=SelectDefaultPen(hDC,i);
	locgraph->draworbit(craftinrmin,hDC, true);
	// label view
	int wpos=W/2;
	int hpos=linespacing;
	TextOut(hDC,wpos, hpos, "View:Minor",10);
}


void EjectFunction::showminorsoiview2(HDC hDC, GRAPH *locgraph, bool hypoorcraft, bool showintercept, int majprojection, int majvieworb, bool showrinc)
{

	// Select the projection
	int projection=vars.getivalue(majprojection);
	VECTOR3 temp={0,-1,0};
	switch (projection){
	case 0:
		locgraph->setprojection(temp);
		break;
	case 1:
		locgraph->setprojection(rmin);
		break;
	}

	// Select the scaling factor
	double scale=0;
	locgraph->setviewscale(scale);
	int orbitstoview=vars.getivalue(majvieworb);
	if (majtarget.isvalid())
	{
		locgraph->setviewscale(majtarget);
	}
	if (hypormaj.isvalid() && orbitstoview==0)
	{
		locgraph->setviewscale(hypormaj);
	}
	if (craftinrmaj.isvalid() && orbitstoview!=1)
	{
		locgraph->setviewscale(craftinrmaj);
	}
	if (orbitstoview==0 || locgraph->getviewscale()==0)
	{// Don't scale to this unless no other scaling has occurred, or user requests it
		locgraph->setviewscale(rmin); 
	}
	DWORD i=5;
	HPEN pen=SelectDefaultPen(hDC,i);
	locgraph->drawplanet(hDC, hmajor);

	//Draw the orbits
	i=2;
	pen=SelectDefaultPen(hDC,i);
	locgraph->draworbit(rmin, hDC, true);
	if (majtarget.isvalid()) locgraph->draworbit(majtarget, hDC, true);
	if (hypormaj.isvalid())
	{
		i=3;
		pen=SelectDefaultPen(hDC,i);
		locgraph->draworbit(hypormaj, hDC, true);
	}
	if (craftinrmaj.isvalid())
	{
		i=1;
		pen=SelectDefaultPen(hDC,i);
		locgraph->draworbit(craftinrmaj, hDC, false);
	}
	// Draw the intercept, if possible
	VECTOR3 crpos, tgtpos, crvel, tgtvel, planecept;
	double intercepttime;
	primary.getpositions(&crpos,&tgtpos);
	primary.getvelocities(&crvel,&tgtvel);
	intercepttime=primary.gettimeintercept();

	if (showintercept)
	{
		i=3;
		pen=SelectDefaultPen(hDC,i);
		locgraph->drawtwovector(hDC, crpos,tgtpos);
		i=5;
		pen=SelectDefaultPen(hDC,i);

		primary.getplanecept(&planecept);
		locgraph->drawvector(hDC,planecept);
	}

	//Label the view
	int wpos=W/2;
	int hpos=linespacing;
	TextOut(hDC,wpos, hpos, "View:Major",10);

	//Show, in text form, various bits of information
	wpos=0;
	hpos=3*linespacing;
	int length;
	char buffer[20];
	if (showintercept)
	{
		VECTOR3 dist=crpos-tgtpos;
		VECTOR3 rvel;
		primary.getrelvel(&rvel);
		VECTOR3 dcorr=rvel*(dotproduct(rvel, dist)/dotproduct(rvel, rvel));
		dist=dist-dcorr;
		double approach=vectorsize(dist);

		// Closest approach
		length=sprintf(buffer, "Cl. App.");
		TextOut(hDC, wpos, hpos, buffer, length);
		hpos+=linespacing;
		TextShow(hDC, " ", wpos, hpos, approach);
		hpos+=linespacing;
		TextShow(hDC,"Enc. V:", wpos, hpos, vectorsize(crvel-tgtvel));
		hpos+=linespacing;
		double arrmjd=oapiTime2MJD(intercepttime);
		length=sprintf(buffer,"Enc. MJD %.2f", arrmjd);
		TextOut(hDC, wpos, hpos, buffer, length);
		hpos+=linespacing+linespacing;
	}
	else
	{
		TextShow(hDC,"Maj. Rad:",wpos,hpos,oapiGetSize(hmajor));
		hpos+=linespacing;
		if (craftinrmaj.isvalid())
		{
			TextShow(hDC,"Craft PeD:", wpos, hpos, craftinrmaj.getpedistance());
			hpos+=linespacing;
			length=sprintf(buffer,"Pe MJD %.2f",(craftinrmaj.getpedeltatime()+craftinrmaj.gettimestamp())/86400+simstartMJD);
			TextOut(hDC,wpos,hpos,buffer, length);
			hpos+=linespacing;
		}
		if (hypormaj.isvalid())
		{
			TextShow(hDC,"Hyp Ped ",wpos,hpos,hypormaj.getpedistance());
			hpos+=linespacing;
			double timeatped=(hypormaj.gettimestamp()+hypormaj.getpedeltatime())/86400+simstartMJD;
			length=sprintf(buffer,"H. Pe MJD %.2f",timeatped);
			TextOut(hDC,wpos,hpos,buffer,length);
			hpos+=linespacing;
		}
	}
		// MJD time to eject
	if (hypormaj.isvalid())
	{
		length=sprintf(buffer, "MJD %.2f", ejectmjd);
		TextOut(hDC, wpos, hpos, buffer, length);
		hpos+=linespacing;
		if (showrinc)
		{
			length=sprintf(buffer, "Hyp. R.Inc:%.4g'", 180/PI*acos(dotproduct(unitise(hypormaj.getplanevector()), unitise(rmin.getplanevector()))));
			TextOut(hDC, wpos, hpos, buffer, length);
			hpos+=linespacing;
		}

	}
	//Periapsis
	if (craftinrmaj.isvalid())
	{
		if (showrinc)
		{
			length=sprintf(buffer, "R.Inc:%.4g'", 180/PI*acos(dotproduct(unitise(craftinrmaj.getplanevector()), unitise(rmin.getplanevector()))));
			TextOut(hDC, wpos, hpos, buffer, length);
			hpos+=linespacing;
		}

	}
}

bool EjectFunction::updateinterception(const ORBIT &hypormaj, const ORBIT &craftinrmaj, const ORBIT &majtarget, bool h_or_c)

// Get the selected intercept, if it's valid
{
	if (h_or_c)
	{
		if (hypormaj.isvalid() && majtarget.isvalid())
		{
			primary.updateintercept(hypormaj,majtarget);
			return true;
		}
	}
	else
	{
		if (majtarget.isvalid() && craftinrmaj.isvalid())
		{
			primary.updateintercept(craftinrmaj, majtarget);
			return true;
		}
	}
	return false;
}

