#define STRICT

/* ===============================================================

  globals - holds global functions for the TransX project

  Copyright Duncan Sharpe 2002-2003

  See file transx.cpp for full license

  ================================================================
  */


#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "intercept.h"
#include "mfdvarhandler.h"
#include "transxstate.h"
#include "graph.h"
#include "transx.h"





struct {
	int mode;
}g_transxMFD;



// ==============================================================
// API interface

DLLCLBK void opcDLLInit (HINSTANCE hDLL)
{
	static char name[10] = "TransferX";
	MFDMODESPEC spec;
	spec.name    = name;
	spec.key     = OAPI_KEY_J;
	spec.msgproc = TransxMFD::MsgProc;
	g_transxMFD.mode = oapiRegisterMFDMode (spec);
}

DLLCLBK void opcDLLExit (HINSTANCE hDLL)
{

	oapiUnregisterMFDMode (g_transxMFD.mode);
}

DLLCLBK void opcCloseRenderViewport()
{
	// transxstate::deletealltransxstate();
	// Orbiter deletes all class objects in the dll when the render window is closed.
	// If this routine is not called, all the transxstate objects are deleted by Orbiter, but the static
	// pointers are left when the render window is closed. This caused a crash when the render window
	// was reopened
	transxstate::setrenderviewport(false);
	transxstate::deletenonactivetransxstate();//Delete any transxstates not used by an MFD
}

DLLCLBK void opcFocusChanged(OBJHANDLE newfocus, OBJHANDLE oldfocus)
{
	transxstate::updatefocusvessel(newfocus);
}


bool SelectVariableBody(void *id, char *str, void *usrdata)

{
	return ((MFDvarbody*)usrdata)->SetVariableBody(str);
}

bool SelectMajorbody(void *id, char *str, void *usrdata)

{
	return ((TransxMFD*)usrdata)->SetMajorBody(str);
}


bool ConfirmReset(void *id, char *str, void *usrdata)
{
	return ((TransxMFD*)usrdata)->SetResetflag(str);
}


bool SelectMinorbody(void *id, char *str, void *usrdata)
{
	return ((TransxMFD*)usrdata)->SetMinorBody(str);
}

bool SelectTargetbody(void *id, char *str, void *usrdata)
{
	return ((TransxMFD*)usrdata)->SetTargetBody(str);
}


void TextShow(HDC hDC,const char *label, int wpos, int hpos, double value)
{
	char index[2]=" ";
	if (fabs(value)>1000)
	{
		value/=1000;
		index[0]='k';
	}
	if (fabs(value)>1000)
	{
		value/=1000;
		index[0]='M';
	}
	if (fabs(value)>1000)
	{
		value/=1000;
		index[0]='G';
	}
	if (fabs(value)>1000)
	{
		value/=1000;
		index[0]='T';
	}
	char buffer[30];
	strcpy(buffer, label);
	char buffer2[20];
	sprintf(buffer2,"%.4g", value);
	strcat(buffer2,index);
	strcat(buffer, buffer2);
	int length=strlen(buffer);
	TextOut(hDC, wpos, hpos, buffer, length);
}


// Standard vector functions
double dotproduct(VECTOR3 vector1, VECTOR3 vector2)
{
	
	double temp=vector1.x*vector2.x+vector1.y*vector2.y+vector1.z*vector2.z;
	return temp;
}

VECTOR3 crossproduct (VECTOR3 vector1, VECTOR3 vector2)
{
	VECTOR3 out;
	out.x=vector1.y*vector2.z-vector1.z*vector2.y;
	out.y=vector1.z*vector2.x-vector1.x*vector2.z;
	out.z=vector1.x*vector2.y-vector1.y*vector2.x;
	return out;
}

// Create vector of length 1
VECTOR3 unitise (VECTOR3 vector)
{
	double scalar= 1/sqrt(vector.x*vector.x+vector.y*vector.y+vector.z*vector.z);
	return vector*scalar;
}

// Find length of vector
double vectorsize(VECTOR3 vector)
{
	double temp=sqrt(vector.x*vector.x+vector.y*vector.y+vector.z*vector.z);
	return temp;
}

void matrixmultiply(MATRIX3 &first, MATRIX3 &second, MATRIX3 *result)
{
	int a,b,c;
	for (a=0;a<3;a++)
	{
		c=3*a;
		for (b=0;b<3;b++)
		{
			result->data[3*a+b]=first.data[c]*second.data[b]+first.data[c+1]*second.data[b+3]+first.data[c+2]*second.data[b+6];
		}
	}
}

void matrixmultiply(MATRIX3 &first, VECTOR3 &second, VECTOR3 *result)
{
	int a,c;
	for (a=0;a<3;a++)
	{
		c=3*a;
		result->data[a]=first.data[c]*second.data[0]+first.data[c+1]*second.data[1]+first.data[c+2]*second.data[2];
	}
}


void getinvrotmatrix(VECTOR3 arot, MATRIX3 *invrotmatrix)//arot not really a vector - see arot defn from vessel struct
{
	
	double tcos=cos(arot.z);
	double tsin=sin(arot.z);
	MATRIX3 z={0,0,0,0,0,0,0,0,1};
	z.m11=z.m22=tcos;
	z.m12=-tsin;
	z.m21=tsin;
	tcos=cos(arot.y);
	tsin=sin(arot.y);
	MATRIX3 y={0,0,0,0,1,0,0,0,0};
	y.m11=y.m33=tcos;
	y.m13=tsin;
	y.m31=-tsin;
	tcos=cos(arot.x);
	tsin=sin(arot.x);
	MATRIX3 x={1,0,0,0,0,0,0,0,0};
	x.m22=x.m33=tcos;
	x.m32=tsin;
	x.m23=-tsin;
	MATRIX3 temp;
	matrixmultiply(z,y,&temp);
	matrixmultiply(temp,x,invrotmatrix);
}

	
