#define STRICT

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "intercept.h"


intercept::intercept()
{
	iceptmethod=1;
	newintercept=true;
	gain=1;
	lasttimecorrection=0;
}

void intercept::resetintercept()
{
	iceptmethod=1;
	newintercept=true;
}

void intercept::improveinterceptstraightline(const ORBIT &craft, const ORBIT &target)
{
	double timeoffset=craft.gettimestamp()-target.gettimestamp();
	newintercept=false;
	double craftcosthi,craftsinthi,targetcosthi,targetsinthi;
	craft.vectortothi(iceptalpha,&craftcosthi,&craftsinthi);
	target.vectortothi(iceptbeta,&targetcosthi,&targetsinthi);
	double crafttimeest=craft.GetTimeToThi(craftcosthi,craftsinthi)+timeoffset;
	double targettimeest=target.GetTimeToThi(targetcosthi,targetsinthi);
	//
	//Now correcting the costhi of the craft to account for the time error
	//
	double craftradius=craft.thitoradius(craftcosthi);
	double craftangle=acos(craftcosthi);
	if (craftsinthi<0) craftangle=-craftangle;
	craftangle+=(targettimeest-crafttimeest)*sqrt(craft.getangmomentum2())/(craftradius*craftradius);
	
	//Get time corrected cos and sin of thi
	craftcosthi=cos(craftangle);
	craftsinthi=sin(craftangle);
	// Get vectors for angles now that time error has been reduced
	craft.thitovectors(craftcosthi,craftsinthi,&icraftpos,&icraftvel);
	target.thitovectors(targetcosthi,targetsinthi,&itargetpos,&itargetvel);
	//Find the (Now reduced) time error again
	crafttimeest=craft.GetTimeToThi(craftcosthi,craftsinthi)+timeoffset;
	targettimeest=target.GetTimeToThi(targetcosthi,targetsinthi);
	//Remove this error linearly from the craft position vector
	icraftpos=icraftpos+icraftvel*(targettimeest-crafttimeest);
	//Find the time to closest approach from here
	//We now have craft and target position and velocity
	//Now use dotproduct to estimate remaining course correction
	VECTOR3 relcraftpos=icraftpos-itargetpos;
	VECTOR3 relcraftvel=icraftvel-itargetvel;
	double timecorrection=-dotproduct(relcraftpos,relcraftvel)/dotproduct(relcraftvel,relcraftvel);
	if (timecorrection*lasttimecorrection<0)
	{//Oscillatory
		gain=gain*0.5;
	}
	else
	{
		gain=gain*1.1;
		if (gain>1) gain=1;
	}
	lasttimecorrection=timecorrection;
	//Now calculate new vectors from the time correction
	icraftpos=icraftpos+icraftvel*timecorrection*gain;
	itargetpos=itargetpos+itargetvel*timecorrection*gain;
	relcraftpos=icraftpos-itargetpos;
	iceptalpha=icraftpos;
	iceptbeta=itargetpos;
	icepttimeoffset=targettimeest+timecorrection*gain;
	itimeintercept=icepttimeoffset+target.gettimestamp();
	if (vectorsize(relcraftpos)*3>vectorsize(icraftpos))//Allows method to switch back if solution is no longer good
	{
		iceptmethod=1;
	}
	return;
}
/*
bool intercept::tryplaneceptvector(const ORBIT &craft, const ORBIT &target)
{
	double tcosthi,tsinthi,ttime;
	VECTOR3 craftpos,craftvel,targetpos,targetvel;
	craft.vectortothi(iplanecept,&tcosthi,&tsinthi);//Get the first vector point
	for (int a=0;a<2;a++)
	{
		craft.thitovectors(tcosthi,tsinthi,&craftpos,&craftvel);
		ttime=craft.GetTimeToThi(tcosthi,tsinthi);//Find arrival time
		target.timetovectors(ttime+timeoffset,&targetpos,&targetvel);
		relpos=craftpos-targetpos;
		if (dotproduct(relpos,relpos)*25<dotproduct(craftpos,craftpos))
		{
			//Close enough - have trial vectors
			icraftpos=craftpos;
			icraftvel=craftvel;
			itargetpos=targetpos;
			itargetvel=targetvel;
			itimeintercept=ttime+timeoffset;
			return true;
		}
		tcosthi=-tcosthi;
		tsinthi=-tsinthi;
	}
	//If we get here, neither solution worked
	return false;
}*/
/*
bool intercept::tryintersections(const ORBIT &craft,const ORBIT &target)
{
	//This routine assumes that the relative inclination is pretty small
	VECTOR3 tmajoraxis,tminoraxis;
	double cosrel,sinrel;
	double factora,factorb,factorc;
	target.getaxes(&tmajoraxis,&tminoraxis);
	craft.vectortothi(&cosrel,&sinrel);//Relative cosine and sin for the two orbits

	//Create intermediate constants for quadratic equation for costhi
	factora=craft.geteccentricity()/craft.getangmomentum2()-target.geteccentricity()*cosrel/target.getangmomentum2();
	factorb=1/craft.getangmomentum2()-1/target.getangmomentum2();
	factorc=-sinrel/target.getangmomentum2();

	//Evaluate expression for square root
	double plusorminus=4*factora*factora*factorb*factorb-4*(factora*factora+factorc*factorc)*(factorb*factorb-factorc*factorc);
	if (plusorminus<0) plusorminus=0;
	plusorminus=sqrt(plusorminus)/2*(factora*factora+factorc*factorc);
	double rootcore=-factora*factorb/(factora*factora+factorc*factorc);
	//These are two possible values of cos thi at intercept
	double roota=rootcore+plusorminus;
	double rootb=rootcore-plusorminus;

	//At the moment there are four possible options, and only two of them are real
}*/



void intercept::updateintercept(const ORBIT &craft, const ORBIT &target)

// Updates the intercept structure holding closest approach between 'craft' and target. 'craft' may be hypothetical or actual
// Only one orbit may be hyperbolic

//To be broken up into submethods next time it's revised
{
	double timeoffset=craft.gettimestamp()-target.gettimestamp();
	iplanecept=crossproduct(craft.getplanevector(),target.getplanevector());
	if (iceptmethod==2)
	{
		improveinterceptstraightline(craft,target);
		return;
	}

	//Our task now is to come up with an initial guess for the intercept that works

	//First option is to check to see whether the plane intercept vector is a good first guess.
	//


	const ORBIT *alpha, *beta;
	bool invert;
	if (target.geteccentricity()>craft.geteccentricity())// Ensure alpha is the more eccentric
	{
		alpha=&target;
		beta=&craft;
		timeoffset=-timeoffset;
		invert=true;
	}
	else
	{
		alpha=&craft;
		beta=&target;
		invert=false;
	}
	double betacos=beta->getcurrcosthi();
	double alphacos=alpha->getcurrcosthi();
	double alphasin;
	if (newintercept)
	{
		iceptradius=beta->getcurrradius();
		newintercept=false;
	}
	double term1=alpha->getangmomentum2()/(alpha->getgmplanet()*alpha->geteccentricity());
	double term2=1/alpha->geteccentricity();
	double timea, timeb;
	VECTOR3 alphaposa, alphaposb, alphavel, betaposa, betaposb, betavela, betavelb;
	alphacos=term1/iceptradius-term2; //This gives the cos of the angle - can be (and frequently is) beyond range of cosine
	double tradius=iceptradius;
	if (alphacos>1) //Make angle legal !
	{
		alphacos=1;
		tradius=fabs(alpha->getangmomentum2()/(alpha->getgmplanet()*(1-alpha->geteccentricity())));
	}
	if (alphacos<-1)
	{
		alphacos=-1;
		tradius=fabs(alpha->getangmomentum2()/(alpha->getgmplanet()*(alpha->geteccentricity()+1)));
	}
	alpha->GetTimesToThi(alphacos, &timea, &timeb); //time a is always sooner - may be negative if hyperbolic
	timea+=timeoffset;
	timeb+=timeoffset;
	alphasin=1-alphacos*alphacos;
	if (alphasin>0)
	{
		alphasin=sqrt(alphasin);
	}
	else
	{
		alphasin=0;
	}
	VECTOR3 majoraxis, minoraxis;
	alpha->getaxes(&majoraxis,&minoraxis);
	alphaposa=majoraxis*(tradius*alphacos)+minoraxis*(tradius*(alphasin));
	alphaposb=majoraxis*(tradius*alphacos)+minoraxis*(tradius*(-alphasin)); //Either pos could match either time at this stage
	if (tradius<alpha->getcurrradius() || alpha->geteccentricity()>1)
	{
		VECTOR3 temp=alphaposa;
		alphaposa=alphaposb;
		alphasin=-alphasin;
		alphaposb=temp;
	}
	// Pos now guaranteed to match
	beta->timetovectors(timea, &betaposa, &betavela); 
	beta->timetovectors(timeb, &betaposb, &betavelb);
	bool abetter=(vectorsize(betaposa-alphaposa)<vectorsize(betaposb-alphaposb)); //Picks best on grounds of distance
	if (timea<timeoffset) abetter=false;//Invalidates first solution if in the past
	if (abetter)
	{
		//a is better
		iceptradius=(vectorsize(betaposa)+iceptradius)/2;// Takes average of new and old as this converges better
		alpha->thitovectors(alphacos, alphasin, &alphaposa, &alphavel);
		itimeintercept=timea;
		if (vectorsize(betaposa-alphaposa)*3<vectorsize(alphaposa))
		{
			iceptmethod=2;
			icepttimeoffset=timea;
			if (invert)
			{
				iceptalpha=betaposa;
				iceptbeta=alphaposa;
			}
			else
			{
				iceptalpha=alphaposa;
				iceptbeta=betaposa;
			}
		}
	}
	else
	{
		//b is better
		iceptradius=(vectorsize(betaposb)+iceptradius)/2;
		alpha->thitovectors(alphacos, -alphasin, &alphaposa, &alphavel);
		itimeintercept=timeb;
		if (vectorsize(betaposb-alphaposa)*3<vectorsize(alphaposa))
		{
			iceptmethod=2;
			icepttimeoffset=timeb;
			if (invert)
			{
				iceptalpha=betaposb;
				iceptbeta=alphaposa;
			}
			else
			{
				iceptalpha=alphaposa;
				iceptbeta=betaposb;
			}
		}
	}
	if (invert)
	{
		itimeintercept-=timeoffset;
		icepttimeoffset=itimeintercept;
	}
	if (target.geteccentricity()>craft.geteccentricity())
	{
		itargetpos=alphaposa;
		itargetvel=alphavel;
		if (abetter)
		{
			icraftpos=betaposa;
			icraftvel=betavela;
		}
		else
		{
			icraftpos=betaposb;
			icraftvel=betavelb;
		}
	}
	else
		{
		icraftpos=alphaposa;
		icraftvel=alphavel;
		if (abetter)
		{
			itargetpos=betaposa;
			itargetvel=betavela;
		}
		else
		{
			itargetpos=betaposb;
			itargetvel=betavelb;
		}
	}
	itimeintercept+=target.gettimestamp();
}

/*
void intercept::updateintercept(const ORBIT &craft, const ORBIT &target)

// Updates the intercept structure holding closest approach between 'craft' and target. 'craft' may be hypothetical or actual
// Only one orbit may be hyperbolic

//To be broken up into submethods next time it's revised
{
	double timeoffset=craft.gettimestamp()-target.gettimestamp();
	iplanecept=crossproduct(craft.getplanevector(),target.getplanevector());
	if (iceptmethod==2)
	{
		improveinterceptstraightline(craft,target);
	}
	const ORBIT *alpha, *beta;
	bool invert;
	if (target.geteccentricity()>craft.geteccentricity())// Ensure alpha is the more eccentric
	{
		alpha=&target;
		beta=&craft;
		timeoffset=-timeoffset;
		invert=true;
	}
	else
	{
		alpha=&craft;
		beta=&target;
		invert=false;
	}
	double betacos=beta->getcurrcosthi();
	double alphacos=alpha->getcurrcosthi();
	double alphasin;
	if (newintercept)
	{
		iceptradius=beta->getcurrradius();
		newintercept=false;
	}
	double term1=alpha->getangmomentum2()/(alpha->getgmplanet()*alpha->geteccentricity());
	double term2=1/alpha->geteccentricity();
	double timea, timeb;
	VECTOR3 alphaposa, alphaposb, alphavel, betaposa, betaposb, betavela, betavelb;
	alphacos=term1/iceptradius-term2; //This gives the cos of the angle - can be (and frequently is) beyond range of cosine
	double tradius=iceptradius;
	if (alphacos>1) //Make angle legal !
	{
		alphacos=1;
		tradius=fabs(alpha->getangmomentum2()/(alpha->getgmplanet()*(1-alpha->geteccentricity())));
	}
	if (alphacos<-1)
	{
		alphacos=-1;
		tradius=fabs(alpha->getangmomentum2()/(alpha->getgmplanet()*(alpha->geteccentricity()+1)));
	}
	alpha->GetTimesToThi(alphacos, &timea, &timeb); //time a is always sooner - may be negative if hyperbolic
	timea+=timeoffset;
	timeb+=timeoffset;
	alphasin=1-alphacos*alphacos;
	if (alphasin>0)
	{
		alphasin=sqrt(alphasin);
	}
	else
	{
		alphasin=0;
	}
	VECTOR3 majoraxis, minoraxis;
	alpha->getaxes(&majoraxis,&minoraxis);
	alphaposa=majoraxis*(tradius*alphacos)+minoraxis*(tradius*(alphasin));
	alphaposb=majoraxis*(tradius*alphacos)+minoraxis*(tradius*(-alphasin)); //Either pos could match either time at this stage
	if (tradius<alpha->getcurrradius() || alpha->geteccentricity()>1)
	{
		VECTOR3 temp=alphaposa;
		alphaposa=alphaposb;
		alphasin=-alphasin;
		alphaposb=temp;
	}
	// Pos now guaranteed to match
	beta->timetovectors(timea, &betaposa, &betavela); 
	beta->timetovectors(timeb, &betaposb, &betavelb);
	bool abetter=(vectorsize(betaposa-alphaposa)<vectorsize(betaposb-alphaposb)); //Picks best on grounds of distance
	if (timea<timeoffset) abetter=false;//Invalidates first solution if in the past
	if (abetter)
	{
		//a is better
		iceptradius=(vectorsize(betaposa)+iceptradius)/2;// Takes average of new and old as this converges better
		alpha->thitovectors(alphacos, alphasin, &alphaposa, &alphavel);
		itimeintercept=timea;
		if (vectorsize(betaposa-alphaposa)*20<vectorsize(alphaposa))
		{
			iceptmethod=2;
			icepttimeoffset=timea;
			if (invert)
			{
				iceptalpha=betaposa;
				iceptbeta=alphaposa;
			}
			else
			{
				iceptalpha=alphaposa;
				iceptbeta=betaposa;
			}
		}
	}
	else
	{
		//b is better
		iceptradius=(vectorsize(betaposb)+iceptradius)/2;
		alpha->thitovectors(alphacos, -alphasin, &alphaposa, &alphavel);
		itimeintercept=timeb;
		if (vectorsize(betaposb-alphaposa)*20<vectorsize(alphaposa))
		{
			iceptmethod=2;
			icepttimeoffset=timeb;
			if (invert)
			{
				iceptalpha=betaposb;
				iceptbeta=alphaposa;
			}
			else
			{
				iceptalpha=alphaposa;
				iceptbeta=betaposb;
			}
		}
	}
	if (invert)
	{
		itimeintercept-=timeoffset;
		icepttimeoffset=itimeintercept;
	}
	if (target.geteccentricity()>craft.geteccentricity())
	{
		itargetpos=alphaposa;
		itargetvel=alphavel;
		if (abetter)
		{
			icraftpos=betaposa;
			icraftvel=betavela;
		}
		else
		{
			icraftpos=betaposb;
			icraftvel=betavelb;
		}
	}
	else
		{
		icraftpos=alphaposa;
		icraftvel=alphavel;
		if (abetter)
		{
			itargetpos=betaposa;
			itargetvel=betavela;
		}
		else
		{
			itargetpos=betaposb;
			itargetvel=betavelb;
		}
	}
	itimeintercept+=target.gettimestamp();
}
*/


void intercept::getplanecept(VECTOR3 *planecept) const
{
	*planecept=iplanecept;
}

void intercept::getpositions(VECTOR3 *craftpos, VECTOR3 *targetpos) const
{
	*craftpos=icraftpos;
	*targetpos=itargetpos;
}

void intercept::getrelpos(VECTOR3 *relpos) const
{
	*relpos=icraftpos-itargetpos;
}

void intercept::getrelvel(VECTOR3 *relvel) const
{
	*relvel=icraftvel-itargetvel;
}

void intercept::getvelocities(VECTOR3 *craftvel, VECTOR3 *targetvel) const
{
	*craftvel=icraftvel;
	*targetvel=itargetvel;
}


double intercept::gettimeintercept() const
{
	return itimeintercept;
}

