#define STRICT
#define ORBITER_MODULE
#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"

#include "mfd.h"
#include "graph.h"
#include "intercept.h"
#include "mfdvarhandler.h"
#include "TransXFunction.h"
#include "transxstate.h"
#include "transx.h"


TransxMFD::TransxMFD (DWORD w, DWORD h, VESSEL *vessel, UINT mfd)
: MFD (w, h, vessel)

// Initialise TransXMFD
{
	//Check to see if new Transxstate is required
	valid=false;
	state=NULL;
	bool success;
	success=transxstate::gettransxstate(mfd,&state,this);
	if (state==NULL)
	{
		//No transxstate, create one
		success=transxstate::newtransxstate(mfd,&state,this);
	}
	if (state==NULL)//Could not create transxstate
	{
		return;
	}
	if (state->isvalid()==false)//functions within transxstate not created
	{
		return;
	}
	valid=true;
	state->setmfdactive(true);
	//state->debug=0;
	graph.setviewwindow(0,0,w,h); //Initialises view window for graphs
	graphleft.setviewwindow(0,0,w/2,h/2);//Left top window
	graphright.setviewwindow(w/2,0,w,h/2);//Right top window
}

TransxMFD::~TransxMFD()
{
	state->setmfdactive(false);
	if (!state->getrenderviewport())//If we're closing, take the state with you, otherwise leave it.
	{
		delete state;//If the view window's closed, delete the state
	}
}

bool TransxMFD::SetTargetBody(char *str)
{
	OBJHANDLE temp=NULL;
	if (strcmp(str,"NT")==0)
	{
		state->GetCurrFunction()->sethmajtarget(temp);
		return true;
	}
	temp=oapiGetObjectByName(str);
	if (temp==NULL) return false;
	state->GetCurrFunction()->sethmajtarget(temp);
	return true;
}



// Called by Orbiter when a screen update is needed
void TransxMFD::Update (HDC hDC)

{
	Title (hDC, "TransX MFD");
	if (!valid) return;
	state->doupdate(hDC,W,H,this);

	//TextShow(hDC,"Debug ",W/2,H/3,state->debug);
	MFDvariable *varpointer=state->GetCurrVariable();
	if (varpointer==NULL) return;
	int linespacing=H/20;
	if (varpointer!=NULL)
	{
		varpointer->show(hDC,W,linespacing);
	}

	
}

int TransxMFD::getwidth()
{
	return W;
}

int TransxMFD::getheight()
{
	return H;
}



int TransxMFD::MsgProc (UINT msg, UINT mfd, WPARAM wparam, LPARAM lparam)

// Standard message parser for messages passed from Orbiter

{
	switch (msg) {
	case OAPI_MSG_MFD_OPENED:
		return (int) new TransxMFD (LOWORD(wparam), HIWORD(wparam), (VESSEL*)lparam, mfd);
	}
	return 0;
}

char *TransxMFD::ButtonLabel (int bt)

// Routine to pass button label back to Orbiter. Called by Orbiter

{
	char *label[12] = {"HLP", "FN", "VW", "MAJ", "MIN", "TGT", "VAR","-VR", "ADJ", "++", "--","RST"};	
	return (bt < 12 ? label[bt] : 0);
}

int TransxMFD::ButtonMenu (const MFDBUTTONMENU **menu) const

// Routine to pass menu description for buttons back to Orbiter. Called by Orbiter

{
	static const MFDBUTTONMENU mnu[12] = {
		{"Toggle Help","On/Off",'H'},
		{"Select Function",0,'F'},
		{"Select view",0,'W'},
		{"Set Major body", 0, 'R'},
		{"Set Minor body", 0, 'M'},
		{"Set Target", 0, 'T'},
		{"Next Variable", 0, 'V'},
		{"Prev. Variable", 0, 'B'},
		{"Set adjustment", "method", 'J'},
		{"Increment", "variable", '='},
		{"Decrement", "variable", '-'},
		{"Reset", 0,'X'}
	};
	if (menu) *menu = mnu;
	return 12;
}

bool TransxMFD::ConsumeKeyImmediate(char *kstate)

// Routine which processes the keyboard state. Allows continuous change of variable.

{
	if (!valid) return false;
	MFDvariable *varpointer=state->GetCurrVariable();
	if (varpointer==NULL) return false;
	if  (*(kstate+OAPI_KEY_MINUS)==-128)
	{
		return varpointer->dec_variable();
	}
	if (*(kstate+OAPI_KEY_EQUALS)==-128)
	{
		return varpointer->inc_variable();
	}
	return false;
}



bool TransxMFD::ConsumeButton(int bt, int event)

// Deal with mouse pressing of keys

{
	static const DWORD btkey[12]={OAPI_KEY_H, OAPI_KEY_F, OAPI_KEY_W, OAPI_KEY_R, OAPI_KEY_M, OAPI_KEY_T, OAPI_KEY_V, OAPI_KEY_B, OAPI_KEY_J, OAPI_KEY_MINUS, OAPI_KEY_EQUALS, OAPI_KEY_X};
	if (!valid) return false;
	MFDvariable *varpointer=state->GetCurrVariable();
	if ((event & PANEL_MOUSE_LBDOWN) && (bt<9 || bt==11)) return ConsumeKeyBuffered (btkey[bt]);
	if (bt>10) return false; //No buttons above 11
	if (varpointer==NULL) return false;
	if ((event & PANEL_MOUSE_LBDOWN) && varpointer->getadjmode()==0)
	{
		// Button 9 or 10 is pressed and the mode is discrete
		return ConsumeKeyBuffered (btkey[bt]);
	}
	// At this point, only the possibility of immediate consumption...
	if (!(event & PANEL_MOUSE_LBPRESSED)) return false; //Mouse button is down on 9 or 10
	if (bt==10)
	{
		varpointer->dec_variable();
	}
	if (bt==9)
	{
		varpointer->inc_variable();
	}
	return true;
}


bool TransxMFD::ConsumeKeyBuffered (DWORD key)

{
	if (!valid) return false;
	MFDvariable *currvar=state->GetCurrVariable();
	bool access=(currvar!=NULL);
	switch (key) {
	case OAPI_KEY_H:
		state->fliphelpsystem();
		return true;
	case OAPI_KEY_F:
		{
			int mfdstate=state->getmfdstate();
			if (++mfdstate>3) mfdstate=1;
			state->setmfdstate(mfdstate);
			state->mfdstatevalidated=false;
		}
		return true;
	case OAPI_KEY_W:
		state->GetCurrFunction()->inc_viewmode();
		return true;
	case OAPI_KEY_R:
		oapiOpenInputBox ("Select Major Reference Body", SelectMajorbody, 0, 20, (void*)this);
		return true;
	case OAPI_KEY_M:
		oapiOpenInputBox ("Select Minor Reference Body", SelectMinorbody, 0, 20, (void*)this);
		return true;
	case OAPI_KEY_T:
		oapiOpenInputBox ("Select Target (NT to clear)", SelectTargetbody, 0, 20, (void*)this);
		return true;
	case OAPI_KEY_X:
		oapiOpenInputBox ("Confirm Reset ?", ConfirmReset, 0, 20, (void*)this);
		return true;
	case OAPI_KEY_V: //Switch variable
		{
			int viewmode=state->GetCurrFunction()->getviewmode();
			state->GetVarhandler()->setnextcurrent(viewmode);
		}
		return true;
	case OAPI_KEY_B:
		{
			int viewmode=state->GetCurrFunction()->getviewmode();
			state->GetVarhandler()->setprevcurrent(viewmode);
		}
		return true;
	case OAPI_KEY_J:
		if (access) currvar->ch_adjmode();
		return true;
	case OAPI_KEY_MINUS:
		if (access)	return currvar->flip_variable();
		return true;
	case OAPI_KEY_EQUALS:
		if (access) return currvar->flip_variable();
		return true;
	}
	return false; //Key not one of cases above
}

bool TransxMFD::SetMajorBody(char *str)
{
	OBJHANDLE temp=oapiGetGbodyByName(str);
	if (temp !=NULL)
	{
		state->GetCurrFunction()->sethmajor(temp);
		return true;
	}
	else
	{
		return false;
	}

}


bool TransxMFD::SetResetflag(char *str)
{
	if (str[0]=='y' || str[0]=='Y') 
	{
		state->GetCurrFunction()->initialisevars();
	}
	return true;
}


bool TransxMFD::SetMinorBody(char *str)
{
	OBJHANDLE temp=oapiGetGbodyByName(str);
	if (temp != NULL)
	{
		state->GetCurrFunction()->sethminor(temp);
		return true;
	}
	else
	{
		return false;
	}
}

void TransxMFD::WriteStatus(FILEHANDLE scn) const
{
	if (!valid) return;
	state->savecurrent(scn);
}


void TransxMFD::ReadStatus(FILEHANDLE scn)
{
	if (!valid) return;
	state->restoresave(scn);
}
