#define STRICT

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "intercept.h"
#include "mfdvarhandler.h"
#include "cruisefunction.h"
#include "orbitejectfunction.h"
#include "transxstate.h"

class transxstate *transxstate::mfdstatelist[20];
bool transxstate::statelistinitflag=false;
bool transxstate::renderviewportopen=true;

TransXFunction *transxstate::GetCurrFunction()
{
	return currfunction;
}

MFDvarhandler *transxstate::GetVarhandler()
{
	return currfunction->getvariablehandler();
}

MFDvariable *transxstate::GetCurrVariable()
{
	return currfunction->getcurrentvariable();
}

bool transxstate::gettransxstate(UINT mfd, class transxstate **state, TransxMFD *tmfdpointer)
{
	if (!statelistinitflag) return false;
	*state=mfdstatelist[mfd];
	if (*state==NULL) return false;
	(*state)->mfdpointer=tmfdpointer;
	return true;
}

class TransxMFD *transxstate::GetMFDpointer()
{
	return mfdpointer;
}

void transxstate::updatefocusvessel(OBJHANDLE newfocus)
{
	for (int a=0;a<20;a++)
	{
		if (mfdstatelist[a]!=NULL)
		{
			mfdstatelist[a]->updateownfocusvessel(newfocus);
		}
	}
}

void transxstate::updateownfocusvessel(OBJHANDLE newfocus)
{
	for (int a=1;a<NUMFUNCTIONS;a++)
	{
		if (functionlist[a]!=NULL) functionlist[a]->sethcraft(newfocus);
	}
}

bool transxstate::newtransxstate(UINT mfd, class transxstate **state, TransxMFD *tmfdpointer)
{
	if (!statelistinitflag)
	{
		for (int a=0;a<20;a++)
		{
			mfdstatelist[a]=NULL;
		}
		statelistinitflag=true;
	}
	if (mfd>19) return false; //Constrain to range
	*state=new transxstate(mfd,tmfdpointer);
	if (*state==NULL) return false;
	return true;
}

void transxstate::deletealltransxstate()
{
	for (int a=0; a<20; a++)
	{
		if (mfdstatelist[a]!=NULL) delete mfdstatelist[a];
	}
}

void transxstate::savecurrent(FILEHANDLE scn)

{
	//Save each function
	for (int a=1;a<NUMFUNCTIONS;a++)//Temporary setting
	{
		if (functionlist[a]!=NULL) functionlist[a]->saveself(scn);
	}
	//Write the current function number
	char buffer[20];//Too big, but who cares!
	strcpy(buffer,"mfdstate");
	oapiWriteScenario_int(scn,buffer,mfdstate);
}

void transxstate::restoresave(FILEHANDLE scn)
{
	//restore settings for each function
	for (int a=1;a<NUMFUNCTIONS;a++)
	{
		if (functionlist[a]!=NULL)
		{
			functionlist[a]->restoreself(scn);
		}
	}
	char *buffer,*member;//Pointer to the buffer assigned by Orbiter
	PARSER parser;
	int tmfdstate,length;
	bool ok=oapiReadScenario_nextline(scn,buffer);
	if (!ok) return;
	parser.parseline(buffer);
	if (parser.getlineelement(1,&member,&length))
	{
		sscanf(member,"%i",&tmfdstate);
	}
	else
	{
		return;
	}
	if (!parser.getlineelement(0,&member,&length)) return;
	if (strcmp(member,"mfdstate")==0 && tmfdstate>0 && tmfdstate<NUMFUNCTIONS)
	{
		oldmfdstate=mfdstate=tmfdstate;//Set the function
		mfdstatevalidated=false;//Just in case it was saved in invalid state
		currfunction=functionlist[mfdstate];
	}
}


transxstate::transxstate(UINT mfd,TransxMFD *tmfdpointer)
{
	if (mfd>=0 && mfd<20)
	{
		mfdstatelist[mfd]=this;
	}
	for (int a=0;a<NUMFUNCTIONS;a++)
	{
		functionlist[a]=NULL;
	}
	valid=initfunctions();//Creates ALL functions
	mfdid=mfd;
	mfdstate=2;
	oldmfdstate=2;
	helpsystem=false;//Switch help system off by default
	mfdactive=true;
	renderviewportopen=true;//

	//To stay
	mfdstatevalidated=false; //True if current mode can be processed. Is always true if MFD switches mode automatically
							//Is initially false after human mode switch. Prevents MFD autoswitching modes when user changes mode
}


void transxstate::deletenonactivetransxstate()
{
	for (int a=0;a<20;a++)
	{
		if (mfdstatelist[a]!=NULL)
		{
			if (mfdstatelist[a]->mfdactive==false)
			{
				delete mfdstatelist[a];
				mfdstatelist[a]=NULL;
			}
		}
	}
}



bool transxstate::initfunctions()
{
	
	//Create some default values for these functions
	OBJHANDLE thmajor,thminor,thtarget,thcraft,thbase;
	thminor=thtarget=thbase=NULL;
	VESSEL *tvessel=oapiGetFocusInterface();
	thmajor=tvessel->GetGravityRef();
	thcraft=oapiGetFocusObject();

	//Initialise functions
	TransXFunction **temp;
	temp=functionlist;
	*temp=NULL;
	temp++;
	*temp=new EjectFunction(this,thmajor,thminor,thtarget,thcraft,thbase);
	if (*temp==NULL) return false;
	temp++;
	*temp=new CruiseFunction(this,thmajor,thminor,thtarget,thcraft,thbase);
	if (*temp==NULL) return false;
	temp++;
	*temp=new OrbitEjectFunction(this,thmajor,thminor,thtarget,thcraft,thbase);
	if (*temp==NULL) return false;
	currfunction=functionlist[2];//Cruise function is default
	return true;
}


transxstate::~transxstate()
{
	if (mfdstatelist[mfdid]!=this)
	{
		//This is a copy, not the controller
		return;
	}
	for (int a=0;a<NUMFUNCTIONS;a++)
	{
		delete functionlist[a];
		functionlist[a]=NULL;
	}
	mfdstatelist[mfdid]=NULL;
}


void transxstate::doupdate(HDC hDC, int tw, int th, TransxMFD *tmfdpointer)
{
	mfdpointer=tmfdpointer;
	int linespacing=th/20;
	if (currfunction==NULL) return; //Just don't do anything if system not initialised
	bool success;
	do
	{
		if (oldmfdstate!=mfdstate)//Switch the mfdstate
		{
			//mfdstate has been changed
			OBJHANDLE thmajor,thminor,thtarget,thcraft,thbase;
			currfunction->gethandles(&thmajor,&thminor,&thtarget, &thcraft,&thbase);//Get handles from old function
			currfunction=functionlist[mfdstate];
			currfunction->sethandles(thmajor,thminor,thtarget,thcraft,thbase);//And pass to the new one
			oldmfdstate=mfdstate;
		}
		//We now have the correct function in place
		if (helpsystem)
		{
			char help1[40],help2[40],help3[40],help4[40],help5[40];
			//Help for variable
			MFDvariable *variable;
			variable=currfunction->getcurrentvariable();
			if (variable!=NULL)
			{
				variable->gethelpstrings(help1,help2);
				TextOut(hDC,0,7*linespacing,help1,strlen(help1));
				TextOut(hDC,0,8*linespacing,help2,strlen(help2));
			}
			//Help for function
			currfunction->gethelp(help1,help2,help3,help4,help5);
			TextOut(hDC,0,14*linespacing,help1,strlen(help1));
			TextOut(hDC,0,15*linespacing,help2,strlen(help2));
			TextOut(hDC,0,16*linespacing,help3,strlen(help3));
			TextOut(hDC,0,17*linespacing,help4,strlen(help4));
			TextOut(hDC,0,18*linespacing,help5,strlen(help5));
			success=true;
		}
		else
		{
			success=currfunction->doupdate(hDC,tw,th);
		}
	} while (!success);
}


int transxstate::getmfdstate() const
{
	return mfdstate;
}

void transxstate::setmfdstate(int tmfd)// Switches functions
{
	mfdstate=tmfd;
}

